(*<sndchips.pas>
 * Types and functions to manage all kind of soundchips supported by the
 * MSX standard.
 * CopyLeft (c) since 1995 by PopolonY2k.
 *)

(**
  *
  * $Id: sndchips.pas 98 2015-08-21 01:28:40Z popolony2k $
  * $Author: popolony2k $
  * $Date: 2015-08-21 01:28:40 +0000 (Fri, 21 Aug 2015) $
  * $Revision: 98 $
  * $HeadURL: file:///svn/p/oldskooltech/code/msx/trunk/msxdos/pascal/sndchips.pas $
  *)

(*
 * This module depends on folowing include files (respect the order):
 * - systypes.pas;
 * - system.pas;
 * - types.pas;
 * - msxbios.pas;
 * - ay8910.pas;
 * - scc.pas;
 * - ym2151.pas;
 * - ym2413.pas;
 * - opl4.pas;
 * - y8950.pas;
 *)

(**
  * The structure containing the sound chip information for all
  * sound chips supported by MSX system.
  *)
Type TSoundChips = Record
  nSCCSlotNumber     : TSlotNumber;          { SCC sound chip handle    }
  nYM2151SlotNumber  : TSlotNumber;          { YM2151 sound chip handle }
  nYM2413SlotNumber  : TSlotNumber;          { YM2413 sound chip handle }
  hasYMF278B         : Boolean;              { YM278B (OPL4) sound chip }
  hasY8950           : Boolean;              { Y8950 sound chip }
  intrTiming         : THostInterruptTiming; { Host interrupt timing }
  nHostTickCount     : Integer;              { Tick counter by host frequency }
End;


(**
  * Initialize all chips used by the loaded VGM.
  * CAUTION: Do not call this function before any other that manipulates
  * memory on any slots because the @see FindSCC function can corrupt data
  * on any slots when it is searching by the SCC interface, so call this
  * function before any other, like @see OpenVGM, in your program;
  * @param chips Reference to a structure that will receive all chips
  * information;
  *)
Procedure InitChips( Var chips : TSoundChips );
Begin
  With chips Do
  Begin
    (* Looking for SCC *)
    nSCCSlotNumber := FindSCC;

    (* Looking for YM2151 *)
    nYM2151SlotNumber := FindYM2151;

    (* Looking for YM2413 *)
    nYM2413SlotNumber := FindYM2413;

    (* Looking for YMF278B *)
    hasYMF278B := FindOPL4;

    (* Looking for Y8950 *)
    hasY8950 := FindY8950;

    (*
     * Chip timming setup.
     *)

    (* Get the host tick count *)
    nHostTickCount := GetInterruptTickCount;

    WriteLn( 'GetInterruptTickCount - PASSED ', nHostTickCount );

    (* Host frequency *)
    intrTiming := GetHostFrequency;
  End;
End;

(**
  * Reset all sound chips specified by the @see TSoundChips structure.
  * @param chips The @see TSoundChips structure containing all sound chips
  * used by the library.
  *)
Procedure ResetChips( Var chips : TSoundChips );
Begin
  With chips Do
  Begin
    (* Reset the AY8910 *)
    ResetAY8910;

    (* Reset YM2413 *)
    If( nYM2413SlotNumber <> 0 )  Then
      ResetYM2413;

    (* Reset SCC *)
    If( nSCCSlotNumber <> 0 )  Then
      ResetSCC( nSCCSlotNumber );

    (* Reset YM2151 *)
    If( nYM2151SlotNumber <> 0 )  Then
      ResetYM2151( nYM2151SlotNumber );

    (* Reset OPL4 *)
    If( hasYMF278B )  Then
      ResetOPL4;

    (* Reset Y8950 *)
    If( hasY8950 )  Then
      ResetY8950;
  End;
End;
